-- ============================================================================
-- == HTCommon - A collection of common functions used throughout the        ==
-- ==            HalonaTools addon.                                          ==
-- ==                                                                        ==
-- ==            This includes message text display... TBC.                  ==
-- ============================================================================

HTCommon           = LibStub("AceAddon-3.0"):NewAddon("HTCommon", "AceConsole-3.0", "AceEvent-3.0")
local L            = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local officerRanks = {}           -- Indicates which ranks are 'officer' ranks.

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTCommon:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTCommon:OnEnable()

  HTCommon:ConfigureOfficerRanks()
  
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTCommon:OnDisable()
end

-- ============================================================================
-- == GetGuildName()                                                         ==
-- ==                                                                        ==
-- == This function retrieves the Guild name of the user, if they are in a   ==
-- == guild, otherwise it returns "Unguilded".                               ==
-- ============================================================================

function HTCommon:GetGuildName()

  if IsInGuild() then
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    return guildName 
  else
    return L["common-guildname-unguilded"]
  end
  
end

-- ============================================================================
-- == GetGuildRankName()                                                     ==
-- ==                                                                        ==
-- == This function retrieves the rank name of the user, if they are in a    ==
-- == guild, otherwise it returns "Unranked".                                ==
-- ============================================================================
 
function HTCommon:GetGuildRankName()

  if IsInGuild() then
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    return guildRankName  
  else
    return L["common-rankname-unguilded"]
  end
  
end

-- ============================================================================
-- ==  GetGuildLevel()                                                       ==
-- ==                                                                        ==
-- == This function retrieves the level of the player's guild.               ==
-- ============================================================================
 
function HTCommon:GetGuildLevel()

  if IsInGuild() then
    return GetGuildLevel()  
  else
    return L["common-rankname-unguilded"]
  end
  
end

-- ============================================================================
-- ==  GetGuildOfficerList()                                                 ==
-- ==                                                                        ==
-- == This function retrieves a list of all "officer" rated characters in    ==
-- == the guild. Note that this will include officer rated alt characters.   ==
-- ============================================================================

function HTCommon:GetGuildOfficerList()

  local guildOfficerList = {}
  
  for index = 1, (GetNumGuildMembers(true)) do
    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
    
    if HTData:SafeNumber(officerRanks[charRankIndex+1]) == 1 then
      table.insert(guildOfficerList, charName)
    end
    
  end     
  
  return guildOfficerList

end

-- ============================================================================
-- ==  ConfigureOfficerRanks()                                               ==
-- ==                                                                        ==
-- == This function configures the 'officerRanks' data table to indicate     ==
-- == which of the ten possible guild ranks currently in use are considered  ==
-- == to be 'Officer Ranks'. This is done by assuming that an 'officer' rank ==
-- == is one which has been granted the privilege of allowing the user the   ==
-- == ability to 'speak' in the officer chat channel.                        ==
-- ==                                                                        ==
-- == The resulting 'officerRanks' table will be a 1 dimensional array       ==
-- == containing 10 indexes where officerRanks[1] corresponds to guild rank  ==
-- == 1 (the 'GM' rank) and officerRanks[10] corresponds to guild rank 10.   ==
-- == If a guild rank is an 'officer' rank then the index holds the value of ==
-- == '1', otherwise the index holds the value of '0' indicating the rank is ==
-- == either not an 'officer' rank or not in use.                            ==
-- ==                                                                        ==
-- == It is expected that this function is called when the addon is loaded   ==
-- == or initialised.                                                        ==
-- ============================================================================

function HTCommon:ConfigureOfficerRanks()

  for i=1,10 do
    
    GuildControlSetRank(i)
    local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote, invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note, modify_guild_info, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags()
    table.insert(officerRanks, HTData:SafeNumericBoolean(officerchat_speak))
    HTConsole:Debug("i=" .. i ..", officer-rank-flag=" .. HTData:SafeNumericBoolean(officerchat_speak))
    
  end

end

-- ============================================================================
-- ==  IsOfficerRank(rankIndex)                                              ==
-- ==                                                                        ==
-- == This function determines whether the supplied rankIndex parameter      ==
-- == represents an "officer-class" guild rank.                              ==
-- ============================================================================

function HTCommon:IsGuildMasterRank(rankIndex)

  if HTData:SafeNumber(rankIndex) == 1 then
    return true
  else
    return false
  end
  
end

-- ============================================================================
-- ==  IsOfficerRank(rankIndex)                                              ==
-- ==                                                                        ==
-- == This function determines whether the supplied rankIndex parameter      ==
-- == represents an "officer-class" guild rank.                              ==
-- ============================================================================

function HTCommon:IsOfficerRank(rankIndex)

  rankIndex = HTData:SafeNumber(rankIndex)
  
  local isOfficerRank = false
  
  if rankIndex < 1 or rankIndex > 10 then
    return isOfficerRank
  end
  
  GuildControlSetRank(rankIndex)
  local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote, invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note, modify_guild_info, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags()
  
  if HTData:SafeNumericBoolean(officerchat_speak) == 1 then
    isOfficerRank = true
  end
  
  return isOfficerRank
  
end

-- ============================================================================
-- ==  IsOfficer(characterName)                                              ==
-- ==                                                                        ==
-- == This function determines whether the character identified by the       ==
-- == supplied name parameter holds an officer level rank within the guild.  == 
-- ============================================================================

function HTCommon:IsOfficer(characterName)

  characterName = HTData:SafeString(characterName)
  
  for index = 1, (GetNumGuildMembers(true)) do
    local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
    
    if charName == characterName then
    
      HTConsole:Debug("CharacterName=" .. charName .. ", officerRanks[" .. charRankIndex .. "+1]=[" .. officerRanks[charRankIndex+1] .. "]")
      
      if HTData:SafeNumber(officerRanks[charRankIndex+1]) == "1" then
        return true
      end      
    end
  end       

  return false
  
end

-- getGuildRankTagList(rankName).
-- hasTag




