-- ============================================================================
-- == HTConsole - A collection of functions related to interactions that     ==
-- ==             occur via the console. This includes most forms of chat    ==
-- ==             based communication and the display of text-messages       ==
-- ==             within Chat windows.                                       ==
-- ============================================================================

HTConsole = LibStub("AceAddon-3.0"):NewAddon("HTConsole", "AceConsole-3.0", "AceEvent-3.0")
local L   = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTConsole:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTConsole:OnEnable()

  self:RegisterEvent("CHAT_MSG_WHISPER")
  
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTConsole:OnDisable()
end

-- ============================================================================
-- == ExecuteChatCommand(arg)                                                ==
-- ==                                                                        ==
-- == This function attempts to execute the supplied argument as though it   ==
-- == were a command entered by the user in the chat window.                 ==
-- ============================================================================

function HTConsole:ExecuteChatCommand(arg)

  if( not DEFAULT_CHAT_FRAME ) then return end
  
  DEFAULT_CHAT_FRAME.editBox:SetText(arg);
  ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox);

end

-- ============================================================================
-- == SendChatMessage(messageText, channelName, language, recipient)         ==
-- ==                                                                        ==
-- == This function is ostensibly a wrapper to the normal SendChatMessage    ==
-- == function with minor changes as follows:                                ==
-- ==                                                                        ==
-- == 1: Language is checked because it may not be passed in.                ==
-- == 2: If channelName = "CHAT", output is routed to self:Print() instead.  ==
-- == 3: If channelName = "NULL", output is muted.                           ==
-- ==                                                                        ==
-- == The intention for the existence of the NULL option for the channel     ==
-- == name is to allow for functions to pass a parameter that can be         ==
-- == switched between a normal channel and null to turn on/off debug mode   ==
-- == features.                                                              ==
-- ============================================================================

function HTConsole:SendChatMessage(messageText, channelName, language, recipient)
  
  -- == -----------------------------------------------------------------------
  -- == -- Check whether the language option has been provided.              --
  -- == -----------------------------------------------------------------------
  if language == nil or language == " " then
    language = nil
  end
  
  channelName = HTConsole:SafeChatChannelName(channelName)
  
  -- == -----------------------------------------------------------------------
  -- == -- Send chat output to 'chat' or to intended channel if not 'NULL'.  --
  -- == -----------------------------------------------------------------------  
  if channelName == "CHAT" then
     print(messageText)
  elseif channelName ~= "NULL" then
     SendChatMessage(messageText, channelName, language, recipient)
  end
  
end

-- ============================================================================
-- == SendWrappedMessage(messageText, channelName)                           ==
-- ==                                                                        ==
-- == This function processes the supplied messageText data into a series of ==
-- == individual messages which are then output to the channel identified by ==
-- == the channelName parameter.                                             ==
-- ============================================================================

function HTConsole:SendWrappedMessage(messageText, channelName)

  local maxWidth        = HTConfig:GetChatWindowTextMaxWidth(nil)
  local words           = HTData:StringSplit(messageText, " ")
  local characterCount  = 0
  local thisLineText    = " "
  local spacerChar      = ""
  
  HTConsole:Debug("maxWidth=[" .. maxWidth .. "]")
  
  -- Loop through the list of words, printing as many per line as the maximum
  -- line width allows.
  for i,v in ipairs(words) do
  
    HTConsole:Debug("Line: [" .. thisLineText .. "], Word: [" .. v .. "], Length: [" .. characterCount .. "], Limit: [" .. maxWidth .. "]")
    if (characterCount + string.len(v)) < maxWidth then
      thisLineText   = thisLineText .. spacerChar .. v
      characterCount = characterCount + string.len(v) + 1
      spacerChar     = " "
    else
      SendChatMessage(thisLineText, channelName, nil, nil)
      thisLineText      = spacerChar .. v .. spacerChar
      characterCount    = 0
      spacerChar        = ""
    end
      
  end
  
  -- Check if there are any words remaining to be printed as a final non-full
  -- line.
  if string.len(thisLineText) > 0 then
    SendChatMessage(thisLineText, channelName, nil, nil)
  end
  
end

-- ============================================================================
-- == SendTitleMessage(messageText, channelName)                             ==
-- ==                                                                        ==
-- == This function processes the supplied messageText data into a series of ==
-- == individual messages which are then output to the channel identified by ==
-- == the channelName parameter.                                             ==
-- ============================================================================

function HTConsole:SendTitleMessage(messageText, channelName)


end

-- ============================================================================
-- == SendTitleBar(channelName)                                              ==
-- ==                                                                        ==
-- == This function outputs a title-bar configured to the width specified in ==
-- == the configuration property describing the maximum chat window width.   ==
-- ============================================================================

function HTConsole:SendTitleBar(channelName)

  local titleBarText = ""
  local maxWidth     = abs(HTConfig:GetChatWindowTextMaxWidth(nil) * 1.5 )
  
  for i=1,maxWidth do
    titleBarText = titleBarText .. "-"  
  end
  
  HTConsole:SendChatMessage(titleBarText, channelName, nil, nil)
  
end


-- ============================================================================
-- == Debug(messageText)                                                     ==
-- ==                                                                        ==
-- == This function outputs the supplied message text to the "CHAT" channel  ==
-- == as a standard format 'debug' message if the debug-mode is currently    ==
-- == enabled.                                                               ==
-- ============================================================================

function HTConsole:Debug(messageText)

  if messageText == nil then return end
  
  if HTConfig:IsFeatureEnabled(L["cfg-feature-name-debugmode"]) then
  
    local messageText = "|cffff0000" .. "[DEBUG]" .. "|r" .. " - " .. messageText
  
    HTConsole:SendChatMessage(messageText, "CHAT", nil, nil)
    
  end

end

-- ============================================================================
-- == SafeChatChannelName(channelName)                                       ==
-- ==                                                                        ==
-- == This function simply checks that the supplied channelName parameter    ==
-- == would be safe to use in the context of this user. If the name would be ==
-- == considered unsafe, a safe alternative is substituted.                  ==
-- ============================================================================

function HTConsole:SafeChatChannelName(channelName)

  -- == -----------------------------------------------------------------------
  -- == -- Setup variables that will be needed.                              --
  -- == -----------------------------------------------------------------------
  --  local playerName = UnitName("player")
  
  -- == -----------------------------------------------------------------------
  -- == -- [GUILD] Checks...                                                 --
  -- == -----------------------------------------------------------------------
  if channelName == "GUILD" then
     if not IsInGuild() then
        channelName = "CHAT"
     end
  end
  
  -- == -----------------------------------------------------------------------
  -- == -- [RAID] Checks...                                                  --
  -- == -----------------------------------------------------------------------
  if channelName == "RAID" then
     if UnitInRaid(playerName) == -1 then
        channelName = "PARTY"
        print("channelName set to Party")
     end
  end
  
  -- == -----------------------------------------------------------------------
  -- == -- [PARTY] Checks...                                                 --
  -- == -----------------------------------------------------------------------
  if channelName == "PARTY" then
     if UnitInParty(playerName) == -1 then
        channelName = "CHAT"
        print("channelName set to Chat")
     end
  end
  
  -- == -----------------------------------------------------------------------
  -- == -- Return the value, adjusted or otherwise, of channelName.          --
  -- == -----------------------------------------------------------------------  
  return channelName
  
end

-- ============================================================================
-- == CHAT_MSG_WHISPER(channel, msg, sender)                                 ==
-- ==                                                                        ==
-- == This function performs actions when the CHAT_MSG_WHISPER event is      ==
-- == triggered. It is triggered when the player's character receives a      ==
-- == message via the whisper 'channel.'                                     ==
-- ============================================================================

function HTConsole:CHAT_MSG_WHISPER(channel, msg, sender)
  
  HTConsole:DoAutoInvitations(msg, sender)

end

-- ============================================================================
-- == DoAutoInvitations(msg, sender)                                         ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text for a recognised request  ==
-- == for a group invitation and sends one automatically when one is found.  ==
-- ============================================================================

function HTConsole:DoAutoInvitations(msg, sender)

  -- == -----------------------------------------------------------------------
  -- == -- Check if the sender was requesting an invite.                     --
  -- == -----------------------------------------------------------------------    
  if HTConsole:IsInvitationKeyword(string.lower(msg)) and HTConfig:IsFeatureEnabled("AutoInvitations") then
    HTConsole:SendChatMessage(L["con-autoinvitations-msg-01"]  , "WHISPER", nil, sender)
    HTConsole:ExecuteChatCommand(L["con-autoinvitations-cmd-01"]..sender)
  end
  
end

-- ============================================================================
-- == IsInvitationKeyword(msg)                                               ==
-- ==                                                                        ==
-- == This function checks the supplied 'msg' text to determine if it        ==
-- == matches one of the recognised invitation keywords.                     ==
-- ============================================================================

function HTConsole:IsInvitationKeyword(msg)

  if msg == L["con-autoinvitations-keyword-01"]
  or msg == L["con-autoinvitations-keyword-02"] then
    return true
  else
    return false
  end
  
end

